// Copyright (C) 2022 - 2023, Xilinx Inc - All rights reserved
//
// Licensed under the Apache License, Version 2.0 (the "License"). You may
// not use this file except in compliance with the License. A copy of the
// License is located at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
// WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
// License for the specific language governing permissions and limitations
// under the License.

#ifndef _XRM_SCALE_INTERFACE_H_
#define _XRM_SCALE_INTERFACE_H_
#ifdef __cplusplus
extern "C" {
#endif

#include <xrm.h>
#include "xrm_interface.h"

typedef struct XrmScaleContext {
  int                 dev_index;
  xrmContext          xrm_ctx;
  int                 xrm_reserve_id;
  int                 scaler_load;
  int                 scaler_res_in_use;
  xrmCuListResourceV2 scaler_cu_list_res;
} XrmScaleContext;

/**
 * xrm_scale_device_deinit: Release and relinquish the xrm resources which
 * were reserved and allocated. Destroy the xrm context API.
 * @param scaler_xrm_ctx: The xrm scaler context
 */
void xrm_scale_release(XrmScaleContext* scaler_xrm_ctx);

/**
 * @brief Check & allocate the necessary xrm resources given the relevant 
 * scaler parameters
 * 
 * @param scaler_xrm_ctx Used to store information about resources this function
 * allocates so they can be safely released later
 * @param dev_index The device index
 * @param input_props Input props used to calculate load
 * @param output_props Output props used to calculate load
 * @param num_outputs Number of outputs in output_props
 * @return XRM_SUCCESS on success, XRM_ERROR on error or XRM_ERROR_CONNECT_FAIL
 * when unable to connect to daemon
 */
int32_t xrm_scale_reserve(XrmScaleContext* scaler_xrm_ctx, int dev_index, XrmInterfaceProperties* input_props, XrmInterfaceProperties* output_props, int num_outputs);
#ifdef __cplusplus
}
#endif
#endif // _XRM_SCALE_INTERFACE_H_
